/**
 * Copyright 2008-2012 Charles Amey
 * 
 * This file is part of Fuel Manager.
 * 
 * Fuel Manager is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Fuel Manager is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Fuel Manager.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "lifetime.h"
#include "mymodel.h"


Lifetime::Lifetime(MainWindow *p)
	: QWidget(p)
{
	parent = p;
	dataTable = new QTableView();
	model = new MyModel();

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(dataTable);
	
	setWindowTitle("All Vehicles Stats");
	setLayout(layout);
	
// 	refreshTable();
// 	testing();
}

Lifetime::~Lifetime()
{
	delete model;
	delete dataTable;
}

void Lifetime::testing()
{
	//set up the colors to rotate by
	//QColor one = Qt::lightGray;
	QColor one(224,229,233);
	QColor two = Qt::white;

	QList<QVariant> r1;
	r1.append("r1c1");
	r1.append("r1c2");
	
	QList<QVariant> r2;
	r2.append("r2c1");
	r2.append("r2c2");
	
	QList< QList<QVariant> > d;
	d.append(r1);
	d.append(r2);
	
	QList<QColor> color;
	color.append(one);
	color.append(two);

	model->setModelData(d,color);
	
	model->setHeaderData(0,Qt::Horizontal,"colname1");
	model->setHeaderData(2,Qt::Horizontal,"colname2");
	
	dataTable->setModel(model);
	dataTable->setAlternatingRowColors(true);
	dataTable->resizeColumnsToContents();
	dataTable->resizeRowsToContents();
}

void Lifetime::refreshTable()
{
	QSqlDatabase c = QSqlDatabase::database(parent->getConnectionName());
	
	//clear out everything first
	model->clearEverything();
	
	//get the currency symbol for the current locale
	QString currencySymbol = QLocale().currencySymbol();
	
	//set up the colors to rotate by
	//QColor one = Qt::lightGray;
// 	QColor one(224,229,233);
// 	QColor two = Qt::white;
	QColor one = parent->getColorOne();
	QColor two = parent->getColorTwo();
	
	if (c.isOpen())
	{
		QSqlQuery query(c);
		QString q = "select description, \
		sum(miles) miles, \
		sum(gallons) gallons, \
		sum(cost) cost \
		from fuel_mileage \
		inner join vehicles \
		on vehicles.id = fuel_mileage.vehicle_id \
		group by fuel_mileage.vehicle_id \
		order by fuel_mileage.vehicle_id \
		";

		if (query.exec(q))
		{
			QSqlRecord rec = query.record();
			int i=0;
			while (i<rec.count())
			{
				model->setHeaderData(i,Qt::Horizontal,rec.fieldName(i));
				i++;
			}
			i++;
			model->setHeaderData(i,Qt::Horizontal,"MPG");
			
			i=0;
			QList< QList<QVariant> > everything;
			QList<QVariant> t;
			QList<QColor> colors;
			double total_miles = 0.0;
			double total_gallons = 0.0;
			double total_cost = 0.0;
			double total_mpg = 0.0;
			while (query.next())
			{
				t.clear();
				for (int i=0;i<query.record().count();i++)
				{
					QVariant c = query.value(i);
					if (c.type() != QVariant::String)
					{
						c = QLocale(QLocale::English).toString(c.toDouble(), 'f', 2);
						if (query.record().fieldName(i)== "cost")
							c = currencySymbol + c.toString();
					}
					t.append(c);
				}
				double mpg = 0.0;
				if (query.value(2) != 0.0)
					mpg = query.value(1).toDouble() / query.value(2).toDouble();
				t.append(mpg);				//mpg
				everything.append(t);
				
				total_miles += query.value(1).toDouble();
				total_gallons += query.value(2).toDouble();
				total_cost += query.value(3).toDouble();
				
				if (i %2 == 0)
					colors.append(two);
				else
					colors.append(one);
				
				i++;
			}
			//save the totals
			t.clear();
			t.append("Total");
			QVariant c = QLocale(QLocale::English).toString(total_miles, 'f', 2);
			t.append(c);
			c = QLocale(QLocale::English).toString(total_gallons, 'f', 2);
			t.append(c);
			c = currencySymbol + QLocale(QLocale::English).toString(total_cost, 'f', 2);
			t.append(c);
			if(total_gallons != 0)
				total_mpg = total_miles / total_gallons;
			c = QLocale(QLocale::English).toString(total_mpg, 'f', 2);
			t.append(c);
			everything.append(t);
			
			model->setModelData(everything,colors);
			dataTable->setModel(model);
			dataTable->setAlternatingRowColors(true);
			dataTable->resizeColumnsToContents();
			dataTable->resizeRowsToContents();
		}
		else
		{
			qDebug()<<query.lastError();
		}
	}
}//end refreshTable